/* (C) 2006 Richard Mills / The Linux Academy Ltd.
 *
 * Provide authentication settings and methods.
 *
 * $Id: AuthSettings.hh,v 1.3 2006-12-02 14:44:10 rich Exp $
 *
 * This file is part of HTTPTunnel.
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _AUTH_SETTINGS_HH_
#define _AUTH_SETTINGS_HH_

#include <string>

namespace HTTPTunnel {

class AuthSettings {
public:

	/*********/
	/* Types */
	/*********/

	// Authorisation schemes
	enum AuthScheme { NONE, UNKNOWN, BASIC, DIGEST };


	/***********/
	/* Methods */
	/***********/

	// Construction/destruction
	AuthSettings(AuthScheme authScheme, std::string realm);
	AuthSettings(AuthScheme authScheme, std::string user, std::string passwd);
	AuthSettings(const AuthSettings& that);
	virtual ~AuthSettings();

	// Assignment
	virtual AuthSettings& operator=(const AuthSettings& that);

	// Construct tunneling request/response headers
	virtual std::string getRequestHeader(std::string uri);
	virtual std::string getResponseHeader(std::string uri);
	std::string getDigestResponse(std::string alg, std::string user,
		std::string passwd, std::string realm, std::string nonce, std::string cnonce,
		std::string cnoncec, std::string qop, std::string uri);

	// Accessors
	AuthScheme getAuthScheme(void) const;
	bool gotUserPasswd(void) const;
	std::string getUser(void) const;
	std::string getPasswd(void) const;

	// Mutators
	void setAuthScheme(AuthScheme scheme);
	void setAuthScheme(AuthScheme scheme, std::string realm);
	void setAuthScheme(AuthScheme scheme, std::string realm, std::string qop);
	void setAuthScheme(AuthScheme scheme, std::string realm, std::string qop,
		std::string nonce);
	void setUserPasswd(std::string user, std::string passwd);
	void newNonce(void);


protected:

	/***********/
	/* Methods */
	/***********/

	// Get authorisation request header field identifier
	virtual std::string getAuthReqestHeaderID(void) const = 0;


	/********/
	/* Data */
	/********/

	// Proxy auth parameters
	AuthScheme						m_authScheme;
	std::string						m_user;
	std::string						m_passwd;
	std::string						m_realm;

	// Basic auth credentials
	std::string						m_credentials;

	// Digest auth credentials
	std::string						m_digestNonce;
	std::string						m_digestQop;
	std::string						m_digestCNonce;
	unsigned int					m_digestCNonceCount;

};


}	// namespace

#endif

