/* (C) 2006 Richard Mills / The Linux Academy Ltd.
 *
 * Client side Tunnel.
 *
 * $Id: ClientTunnel.hh,v 1.7 2007-08-04 11:21:43 rich Exp $
 *
 * This file is part of HTTPTunnel.
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _CLIENT_TUNNEL_HH_
#define _CLIENT_TUNNEL_HH_

#include "Tunnel.hh"

namespace HTTPTunnel {

class ClientTunnel : public Tunnel {
public:
	// Construction/destruction
	ClientTunnel(TunnelConfig* tcfg, Tunneler* tunneler, TunnelState state);
	virtual ~ClientTunnel();


	/************/
	/* Mutators */
	/************/

	// Inform us of how many tunnels in total are currently up
	void tunnelsCount(unsigned int tot);


	/***********/
	/* Methods */
	/***********/

	// Process some newly extracted tunnel data from one of our connections
	virtual void processTunnelMsg(const HTTPMsg* msg);

	// Set whether the tunnel user is authenticated
	void setUserAuthenticated(bool auth);


protected:

	/***********/
	/* Methods */
	/***********/

	// Create a new payload msg for sending down the tunnel
	virtual HTTPMsgParms* makeTunnelMsg(const HTTPMsgParms* recvd,
		unsigned int maxPayload, bool retry) throw(MemoryError);

	// Record that we've sent a new tunnel msg
	virtual void sentNewTunnelMsg(HTTPMsgParms* msgParms);

   // Scale poll timeout
   virtual void scalePollTimeout(void);

	// Have we got a stable HTTP connection to other end?
	virtual bool httpStable(void) const;

	// Can we send a new tunnel msg?
	virtual bool clearToSendNewMsg(Connection* conn);

	// Determine whether an ack implies we should resend a msg
	virtual void checkForResendRequest(const SeqNo& ackNo);

	// Check whether any connections need to be blocked/unblocked
	virtual void checkBlockedConnections(void);


private:

	/*************/
	/* Constants */
	/*************/

	// How many tunnels trigger an increase in poll timeout
	static const unsigned int		TUNNEL_COUNT_POLL_THRESHOLD	= 5;

	// Step size for scale in poll timeout due to no. of tunnels
	static const float				TUNNEL_COUNT_POLL_STEP			= 0.2;


	/********/
	/* Data */
	/********/

	// Number of currently active tunnels
	unsigned int				m_noActiveTunnels;

	// Flag to indicate if we have a clear & stable HTTP connection
	bool							m_httpConnStable;

	// Last received ack sequence no.
	SeqNo							m_lastRecvAckSeqNo;

};


}	// namespace

#endif

