/* (C) 2005 Richard Mills / The Linux Academy Ltd.
 *
 * An event to record a fatal error on some connection.
 *
 * $Id: ConnectionErrorEvent.hh,v 1.5 2007-02-09 16:55:51 rich Exp $
 *
 * This file is part of HTTPTunnel.
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _CONNECTION_ERROR_EVENT_HH_
#define _CONNECTION_ERROR_EVENT_HH_

#include <string>
#include "Event.hh"
#include "IdentifyEvent.hh"
#include "Tunnel.hh"

namespace HTTPTunnel {

class Tunneler;

class ConnectionErrorEvent : public Event, public IdentifyEvent<unsigned int> {
public:
	// Construction/destruction
	ConnectionErrorEvent(unsigned int connID, Tunnel::TunnelErrorType type,
		Tunneler* tunneler);
	virtual ~ConnectionErrorEvent();

	// Display event details for debugging
	virtual void details(std::stringstream& dbg);

	// Accessors
	unsigned int getConnectionID(void) const;
	Tunnel::TunnelErrorType getErrorType(void) const;

	// Run this event
	virtual void execute(void);

protected:

	/********/
	/* Data */
	/********/

	// ID of connection error is from
	unsigned int					m_connID;

	// Type of the error
	Tunnel::TunnelErrorType		m_type;

	// Controlling tunneler
	Tunneler*						m_tunneler;

};


}	// namespace

#endif

