/* (C) 2005 Richard Mills / The Linux Academy Ltd.
 *
 * An object capable of receiving and executing events.
 *
 * $Id: EventReceiver.hh,v 1.6 2006-07-08 13:49:47 rich Exp $
 *
 * This file is part of HTTPTunnel.
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _EVENT_RECEIVER_HH_
#define _EVENT_RECEIVER_HH_

#include <vector>
#include "ConditionVar.hh"
#include "Mutex.hh"
#include "ThreadManager.hh"
#include "Thread.hh"
#include "Exceptions.hh"


namespace HTTPTunnel {

class Event;
class TimedEvent;

class EventReceiver {
public:
	// Construction/destruction
	EventReceiver();
	virtual ~EventReceiver();

	// Accessors
	bool shuttingDown(void) const;


	/***********/
	/* Methods */
	/***********/

	// Schedule a new event
	void scheduleEvent(Event* event);

	// Cancel events of some type
	template<class T, class I> void cancelEvents(I identity);

	// Return whether some event is currently scheduled
	template<class T, class I> bool isScheduled(I identity);


protected:

	/***********/
	/* Methods */
	/***********/

	// Instruct the event scheuling threads to terminate
	void shutdownSchedulers(void);


	/********/
	/* Data */
	/********/

	// A thread manager
	Threads::ThreadManager			m_threadMgr;


private:

	/***********/
	/* Methods */
	/***********/

	// Process events on the event queue - Thread entry point
	void processEvents(void);

	// Shutdown the schedulers
	void shutdownMainScheduler(void);

	// Find events of a particular type matching some identity - Caller holds mutex
	template<class T, class I>
	std::vector<Event*>::iterator findEvent(I identity);
	

	/********/
	/* Data */
	/********/

	// Event processing thread pointers
	Threads::Thread*					m_eventThread;

	// An event queue + condition var & mutex
	std::vector<Event*>				m_events;
	Threads::ConditionVar			m_eventsCond;
	Threads::Mutex						m_eventsMtx;
	bool									m_eventsRunning;

	// Mutex + CondVar to signal when scheduler shutdown is complete
	Threads::Mutex						m_shutdownMtx;
	Threads::ConditionVar			m_shutdownCond;

};


}	// namespace


// Include template methods
#include "EventReceiverT.cc"


#endif

