/* (C) 2007 Richard Mills / The Linux Academy Ltd.
 *
 * An event to flush a connections data buffer.
 *
 * $Id: FlushConnectionEvent.cc,v 1.1 2007-05-20 15:04:16 rich Exp $
 *
 * This file is part of HTTPTunnel.
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "FlushConnectionEvent.hh"
#include "Debug.hh"
#include "Exceptions.hh"
#include "Logger.hh"
#include "Tunneler.hh"


// Reference to global logger
extern HTTPTunnel::Logger* g_pLogger;


namespace HTTPTunnel {


// Construction/destruction
FlushConnectionEvent::FlushConnectionEvent(const Time& time, unsigned int connID,
	bool retry, Tunneler* tunneler)
: TimedEvent(time), m_tunneler(tunneler), m_connID(connID), m_retry(retry)
{
}

FlushConnectionEvent::~FlushConnectionEvent()
{
}


// Display event details for debugging
void FlushConnectionEvent::details(std::stringstream& dbg)
{
	dbg << "FlushConnectionEvent: ";
	TimedEvent::details(dbg);
}


// Run the event
void FlushConnectionEvent::execute(void)
{
	m_tunneler->flushConnection(m_connID, m_retry);
}


}	// namespace

