/* (C) 2007 Richard Mills / The Linux Academy Ltd.
 *
 * Encapsulate resolved host IP addresses.
 *
 * $Id$
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _HOST_ADDR_HH_
#define _HOST_ADDR_HH_

#include <string>
#include <vector>
#include <netdb.h>
#include <arpa/inet.h>
#include "RawBuff.hh"
#include "Exceptions.hh"

namespace HTTPTunnel {

class HostAddr {
public:
	// Construction/destruction
	HostAddr(struct hostent* phost) throw(MemoryError);
	HostAddr(std::string addrStr) throw(MemoryError);
	~HostAddr();

	// Accessors
	bool resolved(void) const;
	const struct in_addr* getAddr(unsigned int no = 0);
	std::string getAddrStr(unsigned int no = 0);


private:

	/*************/
	/* Constants */
	/*************/

	// How large does the buffer need to be for getHostByName()
	static const unsigned int		AUX_HOST_BUFF_SIZE	= 1024;


	/***********/
	/* Methods */
	/***********/

	// Forbid assignment/copy-construction
	HostAddr(const HostAddr& that);
	HostAddr& operator=(const HostAddr& that);

	// Resolve an address to a host entry
	int getHostByName(std::string addr, struct hostent* host);


	/********/
	/* Data */
	/********/

	// List of resolved addresses
	std::vector<RawBuff*>			m_addresses;

	// Got at least one resolved address?
	bool									m_resolved;

};


}	// namespace

#endif

