/* (C) 2003 Kevin Manning.
 * (C) 2005 Richard Mills / The Linux Academy Ltd.
 *
 * Implement a pthreads mutex.
 *
 * $Id: Mutex.cc,v 1.1 2006-04-01 14:11:05 rich Exp $
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <iostream>
#include <pthread.h>
#include <errno.h>
#include "Mutex.hh"

namespace Threads {

// Construction/destruction
Mutex::Mutex() throw(MutexError)
   : m_locked(false)
{
   #ifdef DEBUG_THREADS
   //std::cerr << "Mutex::Mutex()" << std::endl;
   #endif

   // Set attribute for recursive mutex
   if(pthread_mutexattr_init(&m_attr) != 0)
      throw MutexError("pthread_mutexattr_init() failed");
   if(pthread_mutexattr_settype(&m_attr, PTHREAD_MUTEX_RECURSIVE) != 0)
      throw MutexError("pthread_mutexattr_settype() failed");

   if(pthread_mutex_init(&m_mutex, &m_attr) != 0)
      throw MutexError("pthread_mutex_init() failed");
}

Mutex::~Mutex()
{
   #ifdef DEBUG_THREADS
   //std::cerr << "Mutex::~Mutex()" << std::endl;
   #endif

   pthread_mutex_destroy(&m_mutex);
}


// Lock the mutex
void Mutex::lock(void) throw(MutexError)
{
   #ifdef DEBUG_THREADS
   //std::cerr << "Mutex::lock()" << std::endl;
   #endif

   if(pthread_mutex_lock(&m_mutex) != 0)
      throw MutexError("pthread_mutex_lock() failed");

   // Now locked
   m_locked = true;
}

// Unlock the mutex
void Mutex::unlock(void) throw(MutexError)
{
   #ifdef DEBUG_THREADS
   //std::cerr << "Mutex::unlock()" << std::endl;
   #endif

   if(pthread_mutex_unlock(&m_mutex) != 0)
      throw MutexError("pthread_mutex_unlock() failed");

   // Now unlocked
   m_locked = false;
}

// Try to lock the mutex
bool Mutex::tryLock(void)
{
   if(pthread_mutex_trylock(&m_mutex) == EBUSY)
      return false;

   // Now locked
   m_locked = true;

   return true;
}

// Is the mutex locked ?
bool Mutex::isLocked(void) const
{
   #if 0
   if(pthread_mutex_trylock(&m_mutex) == EBUSY)
      return true;

   // Mutex wasn't locked, reset it to unlocked
   if(pthread_mutex_unlock(&m_mutex) != 0)
      throw MutexError("pthread_mutex_unlock() failed");
   return false;
   #endif

   return m_locked;
}

// Return ptr to mutex we wrap
pthread_mutex_t* Mutex::getMtx(void)
{
   return &m_mutex;
}


}	// namespace Threads

