/* (C) 2006 Richard Mills / The Linux Academy Ltd.
 *
 * Parse options from either the command line or a config file.
 *
 * $Id: OptionsParser.hh,v 1.6 2007-07-01 15:39:03 rich Exp $
 *
 * This file is part of HTTPTunnel.
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _OPTIONS_PARSER_HH_
#define _OPTIONS_PARSER_HH_

#include <string>
#include "Exceptions.hh"
#include "Enums.hh"

namespace HTTPTunnel {

class OptionsParser {
public:
	// Construction/destruction
	OptionsParser();
	~OptionsParser();

	// Parse the contents of a config file
	void parseCfgFile(const char* name) throw(ConfigError);


private:

	/***********/
	/* Methods */
	/***********/

	// Check and get numeric value
	unsigned int getNumeric(const std::string& from, std::string err) const
		throw(ConfigError);

	// Check string formatting
	bool isNumeric(const std::string& str) const;

	// Parse section lines
	void parseGlobal(std::string line) throw(ConfigError);
	void parseClient(std::string line) throw(ConfigError);
	void parseServer(std::string line) throw(ConfigError);

	// Use parsed options for a section to create runtime config
	void sectionFinished(bool global, bool client, bool server)
		throw(ConfigError);


	/********/
	/* Data */
	/********/

	// Parsed general config
	std::string					m_localIP;
	unsigned int				m_localPort;
	std::string					m_sslCert;
	std::string					m_sslCertKey;

	// Parsed client config
	bool							m_cliFixed;
	bool							m_cliSOCKS;
	bool							m_cliProxy;
	Enums::Protocol			m_cliProtocol;
	std::string					m_cliServerIP;
	unsigned int				m_cliServerPort;
	bool							m_cliSSL;
	std::string					m_cliProxyIP;
	unsigned int				m_cliProxyPort;
	std::string					m_cliProxyUser;
	std::string					m_cliProxyPasswd;
	std::string					m_cliDestIP;
	unsigned int				m_cliDestPort;
	std::string					m_cliAuthUser;
	std::string					m_cliAuthPasswd;

	// Parsed server config
	std::string					m_svrFirewallIPs;
	std::string					m_svrFirewallPorts;
	bool							m_svrAuth;
	bool							m_svrSSL;

};


}	// namespace

#endif

