/* (C) 2006 Richard Mills / The Linux Academy Ltd.
 *
 * Encapsulate settings for tunneling through a proxy.
 *
 * $Id: ProxySettings.cc,v 1.9 2006-08-28 12:05:49 rich Exp $
 *
 * This file is part of HTTPTunnel.
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <iostream>
#include <sstream>
#include <iomanip>
#include "ProxySettings.hh"


namespace HTTPTunnel {


// Construction/destruction
ProxySettings::ProxySettings(std::string phost, unsigned int pport,
	AuthScheme authScheme, std::string user, std::string passwd)
: AuthSettings(authScheme, user, passwd), m_IP(phost), m_port(pport)
{
}

ProxySettings::ProxySettings(const ProxySettings& that)
: AuthSettings(that), m_IP(that.m_IP), m_port(that.m_port)
{
}

ProxySettings::~ProxySettings()
{
}


// Assignment
ProxySettings& ProxySettings::operator=(const ProxySettings& that)
{
	// Self assignment
	if(this != &that) {
		AuthSettings::operator=(that);
		m_IP = that.m_IP;
		m_port = that.m_port;
	}

	return *this;
}


/*************/
/* Utilities */
/*************/


// Construct tunneling proxy headers
std::string ProxySettings::getRequestHeader(std::string uri)
{
	std::stringstream res;

	// General proxy fields
	res << "Proxy-Connection: keep-alive\r\n";

	// Proxy authentication fields
	res << AuthSettings::getRequestHeader(uri);

	return res.str();
}


/*************/
/* Accessors */
/*************/


// Get proxy IP
std::string ProxySettings::getIP(void) const
{
	return m_IP;
}

// Get proxy port
unsigned int ProxySettings::getPort(void) const
{
	return m_port;
}


/***********************/
/* Protected Interface */
/***********************/


// Get authorisation request header field identifier
std::string ProxySettings::getAuthReqestHeaderID(void) const
{
	return "Proxy-Authorization";
}


}	// namespace

