/* (C) 2006 Richard Mills / The Linux Academy Ltd.
 *
 * Encapsulate settings for tunneling through a proxy.
 *
 * $Id: ProxySettings.hh,v 1.8 2006-08-28 12:05:49 rich Exp $
 *
 * This file is part of HTTPTunnel.
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _PROXY_SETTINGS_HH_
#define _PROXY_SETTINGS_HH_

#include <string>
#include "AuthSettings.hh"

namespace HTTPTunnel {

class ProxySettings : public AuthSettings {
public:
	// Construction/destruction
	ProxySettings(std::string phost, unsigned int pport, AuthScheme authScheme,
		std::string user, std::string passwd);
	ProxySettings(const ProxySettings& that);
	virtual ~ProxySettings();

	// Assignment
	virtual ProxySettings& operator=(const ProxySettings& that);

	// Construct tunneling proxy headers
	virtual std::string getRequestHeader(std::string uri);

	// Accessors
	std::string getIP(void) const;
	unsigned int getPort(void) const;


protected:

	/***********/
	/* Methods */
	/***********/

	// Get authorisation request header field identifier
	virtual std::string getAuthReqestHeaderID(void) const;


	/********/
	/* Data */
	/********/

	// Proxy IP address/port
	std::string						m_IP;
	unsigned int					m_port;

};


}	// namespace

#endif

