/* (C) 2006 Richard Mills / The Linux Academy Ltd.
 *
 * Encapsulate settings for server authentication.
 *
 * $Id: ServerAuthSettings.cc,v 1.2 2006-08-28 12:05:49 rich Exp $
 *
 * This file is part of HTTPTunnel.
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <iostream>
#include <sstream>
#include <iomanip>
#include "ServerAuthSettings.hh"


namespace HTTPTunnel {


// Construction/destruction
ServerAuthSettings::ServerAuthSettings(AuthScheme authScheme, std::string realm)
: AuthSettings(authScheme, realm)
{
}

ServerAuthSettings::ServerAuthSettings(AuthScheme authScheme, std::string user,
	std::string passwd)
: AuthSettings(authScheme, user, passwd)
{
}

ServerAuthSettings::ServerAuthSettings(const ServerAuthSettings& that)
: AuthSettings(that)
{
}

ServerAuthSettings::~ServerAuthSettings()
{
}


// Assignment
ServerAuthSettings& ServerAuthSettings::operator=(const ServerAuthSettings& that)
{
	// Self assignment
	if(this != &that) {
		AuthSettings::operator=(that);
	}

	return *this;
}


/************/
/* Utilties */
/************/


// Construct tunneling server response header
std::string ServerAuthSettings::getResponseHeader(std::string uri)
{
	std::stringstream res;

	// Using authorization?
	if(m_authScheme == BASIC) {
		// Set basic authentication fields
		res << "WWW-Authenticate: Basic realm=\"" << m_realm << "\"\r\n";
	}
	else if(m_authScheme == DIGEST) {
		// Set digest authentication fields
		res << "WWW-Authenticate: Digest realm=\"" << m_realm << "\", ";
		res << "nonce=\"" << m_digestNonce << "\", ";
		res << "qop=\"" << m_digestQop << "\", ";
		res << "stale=false\r\n";
	}

	return res.str();
}


/***********************/
/* Protected Interface */
/***********************/


// Get authorisation request header field identifier
std::string ServerAuthSettings::getAuthReqestHeaderID(void) const
{
	return "Authorization";
}


}	// namespace

