/* (C) 2007 Richard Mills / The Linux Academy Ltd.
 *
 * An event to record the result of a server test connect attempt.
 *
 * $Id: ServerTestConnectResultEvent.cc,v 1.1 2007-07-07 10:56:02 rich Exp $
 *
 * This file is part of HTTPTunnel.
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "ServerTestConnectResultEvent.hh"
#include "ClientTunneler.hh"
#include "Debug.hh"

namespace HTTPTunnel {


// Construction/destruction
ServerTestConnectResultEvent::ServerTestConnectResultEvent(Tunneler* tunneler,
	unsigned int connID, std::string host, unsigned int port, Enums::Protocol prot,
	bool ssl, std::string cert, ConnectResultType type)
: ConnectResultEvent(tunneler, connID, host, port, prot, ssl, cert, type)
{
}

ServerTestConnectResultEvent::~ServerTestConnectResultEvent()
{
}


// Display event details for debugging
void ServerTestConnectResultEvent::details(std::stringstream& dbg)
{
	dbg << "ServerTestConnectResultEvent: ";
	ConnectResultEvent::details(dbg);
}


// Report the error
void ServerTestConnectResultEvent::execute(void)
{
	DPRINTV("ServerTestConnectResultEvent::execute()");

	// Let controlling tunneler know
	ClientTunneler* cliTunneler = dynamic_cast<ClientTunneler*>(m_tunneler);
	cliTunneler->serverTestConnectResult(this);
}


}	// namespace

