/* (C) 2006 Richard Mills / The Linux Academy Ltd.
 *
 * Server side Tunnel.
 *
 * $Id: ServerTunnel.hh,v 1.7 2007-08-04 11:21:43 rich Exp $
 *
 * This file is part of HTTPTunnel.
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _SERVER_TUNNEL_HH_
#define _SERVER_TUNNEL_HH_

#include <vector>
#include <string>
#include "Tunnel.hh"

namespace HTTPTunnel {

class ServerTunnel : public Tunnel {
public:
	// Construction/destruction
	ServerTunnel(TunnelConfig* tcfg, Tunneler* tunneler, TunnelState state);
	virtual ~ServerTunnel();


	/*************/
	/* Accessors */
	/*************/

	// Get source/destination addresses of connections for this tunnel
	virtual std::string getSrcConnectionAddresses(void);
	virtual std::string getDstConnectionAddresses(void);


	/***********/
	/* Methods */
	/***********/

	// Process some newly extracted tunnel data from one of our connections
	virtual void processTunnelMsg(const HTTPMsg* msg);


protected:

	/***********/
	/* Methods */
	/***********/

	// Create a new payload msg for sending down the tunnel
	virtual HTTPMsgParms* makeTunnelMsg(const HTTPMsgParms* recvd,
		unsigned int maxPayload, bool retry) throw(MemoryError);

	// Record that we've sent a new tunnel msg
	virtual void sentNewTunnelMsg(HTTPMsgParms* msgParms);

	// Can we send a new tunnel msg?
	virtual bool clearToSendNewMsg(Connection* conn);

	// Determine whether an ack implies we should resend a msg
	virtual void checkForResendRequest(const SeqNo& ackNo);

	// Check whether any connections need to be blocked/unblocked
	virtual void checkBlockedConnections(void);


private:

	/*************/
	/* Constants */
	/*************/

	// Minimum client version we allow to connect
	static const unsigned int		MIN_CLIVER_MAJOR		= 0;
	static const unsigned int		MIN_CLIVER_MINOR		= 9;
	static const unsigned int		MIN_CLIVER_POINT		= 4;


	/***********/
	/* Methods */
	/***********/

	// Try to authenticate a client
	void authenticateClient(const std::vector<std::string>& authArgs);

	// Is this tunnel user authenticated?
	bool userAuthenticated(void);

};


}	// namespace

#endif

