/* (C) 2005 Richard Mills / The Linux Academy Ltd.
 *
 * Tunnel controller for server end of tunnel.
 *
 * $Id: ServerTunneler.hh,v 1.36 2007-06-16 10:43:56 rich Exp $
 *
 * This file is part of HTTPTunnel.
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _SERVER_TUNNELER_HH_
#define _Server_TUNNELER_HH_

#include <vector>
#include "Tunneler.hh"
#include "Exceptions.hh"

namespace HTTPTunnel {

class Tunnel;
class Connection;
class HTTPMsg;
class ServerSideConfig;


class ServerTunneler : public Tunneler {
public:
   // Construction/destruction
   ServerTunneler(Server* server);
   virtual ~ServerTunneler();


	/**********/
	/* Events */
	/**********/

	// Notification of a new socket connection
	virtual void newConnectionArrived(Sock* insock) throw(CommsError);

	// Notification that a tunnel has insufficient left/right connections to be viable
	virtual void unviableTunnel(const TunnelID& tid);

	// Notification of new HTTP message arrival
	virtual void messageArrived(unsigned int connID, const HTTPMsg* msg)
		throw(LookupManagementError);

	// Notification of new tunneled/raw data arrival
	virtual void rawDataArrived(unsigned int connID, const TunnelID& tid,
		const RawBuff& raw) throw(LookupManagementError);

	// Check we have received an ack for some sent message
	virtual void checkAckReceived(void);


protected:

	/***********/
	/* Methods */
	/***********/

	// The number of tunnels in existance has changed
	virtual void noTunnelsChanged(void);


private:

	/*************/
	/* Constants */
	/*************/

	// Timeout for client reconnecting a broken tunnel
	static const unsigned int		TUNNEL_RECONNECT_TIMEOUT		= 30;


	/***********/
	/* Methods */
	/***********/

	// Create a new carried tunnel in response to a received message
	Tunnel* newTunnelConnectionRequest(Connection* conn, TunnelID* tid,
		const HTTPMsgParms* parms);

	// Reconfigure an existing tunnel in response to a reconnection message
	void tunnelReconnectionRequest(Tunnel* tun, Connection* conn,
		Tunnel::TunnelState reportedState);

	// Reconfigure an existing tunnel in response to a new pipe connection
	void newTunnelPipeConnectionRequest(Tunnel* tun, Connection* conn);

	// Handle an authentication response from a client
	void authenticationResponse(Tunnel* tun, const TunnelID& tid);

	// Send back a tunnel error response to a client
	void sendFatalErrorResponse(Connection* conn, const HTTPMsg* msg);

};


}	// namespace

#endif

