/* (C) 2003 Kevin Manning.
 * (C) 2005 Richard Mills / The Linux Academy Ltd.
 *
 * Encapsulate the functionality of a thread.
 *
 * $Id: Thread.hh,v 1.2 2007-04-17 13:44:05 rich Exp $
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _THREAD_HH_
#define _THREAD_HH_

#include <pthread.h>
#include "Mutex.hh"
#include "Callback.hh"
#include "ThreadExceptions.hh"

namespace Threads {

// Forward declarations
class ThreadManager;

class Thread {
public:
	// Construction/destruction
	Thread(ThreadManager* manager, Callback& cbf, unsigned int tc)
		throw(ThreadError);
	~Thread();

	// Operations
	void start(void);

	// Accessors
	unsigned int getThreadCount(void) const;

private:

	// Prevent copy construction / assignment
	Thread(const Thread&);
	Thread& operator=(const Thread&);

	// Starting point for all new threads
	static void* threadMain(void* args);

	// Tidy up before exit
	static void tidyUp(void* args);

	// Our manager
	ThreadManager*			m_manager;
	// Our callback function
	Callback*				m_callback;
	// Our pthread identifier
	pthread_t				m_thread;

	// Our mutex
	Mutex						m_mtx;

	// Our thread pool id
	unsigned int			m_tc;

};

}		// namespace Threads

#endif

