/* (C) 2003 Kevin Manning.
 * (C) 2005 Richard Mills / The Linux Academy Ltd.
 *
 * Thread Manager class.
 *
 * $Id: ThreadManager.hh,v 1.2 2007-04-17 13:44:05 rich Exp $
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _THREAD_MANAGER_HH_
#define _THREAD_MANAGER_HH_

#include <pthread.h>
#include <vector>
#include "ThreadExceptions.hh"
#include "Thread.hh"
#include "Callback.hh"
#include "Mutex.hh"
#include "ConditionVar.hh"

namespace Threads {

// A wrapper for thread details
class ThreadDetails {
public:
	ThreadDetails(Thread* thread);
	~ThreadDetails();

	// Get/set thread
	Thread* getThread(void) const;
	void setThread(Thread* thread);

private:

	// The thread these details are for
	Thread*			  m_thread;

};


// A thread manager
class ThreadManager {

	friend class Thread;

public:
	enum { MAX_THREADS = 16384 };

	// Construction/Destruction
	ThreadManager(int maxThreads = MAX_THREADS) throw(MutexError);
	~ThreadManager();

	// Thread management
	Thread* newThread(Callback& callBack) throw(ThreadError);

private:

	// Predicate for looking up thread details from thread
	struct PredGetDetails {
		PredGetDetails(Thread* thread) : m_thread(thread) {}
		bool operator()(const ThreadDetails* details) const {
			return (details->getThread() == m_thread);
		}
		Thread* m_thread;
	};

	// Callbacks for thread to let us know how it's doing
	void started(Thread* thread);
	void finished(Thread* thread);
	void waitForAll(void);

	// The set of managed threads
	std::vector<ThreadDetails*>	m_threadDetails;

	// The max. no. of threads we will manage
	unsigned int					m_maxThreads;

	// Local mutex
	Mutex								m_mtx;

	// Condition var for thread to let us know it's finished
	ConditionVar					m_cv_thread_finished;

	/* Static data */

	// Global mutex to protect access to static data
	static Mutex					c_mtx;

	// Pool of thread id's
	static bool						c_poolIds[MAX_THREADS];

};

}		// namespace Threads

#endif


