/* (C) 2005 Richard Mills / The Linux Academy Ltd.
 *
 * Uniquely identify an individual tunneled connection.
 *
 * $Id: TunnelID.hh,v 1.10 2007-04-29 11:56:38 rich Exp $
 *
 * This file is part of HTTPTunnel.
 *
 * HTTPTunnel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * HTTPTunnel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HTTPTunnel; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _TUNNEL_ID_HH_
#define _TUNNEL_ID_HH_

#include <string>
#include <iostream>
#include <vector>
#include "Exceptions.hh"
#include "Enums.hh"

namespace HTTPTunnel {

class TunnelID {
public:
	// Construction/destruction
	TunnelID();
	TunnelID(unsigned int id, std::string serverip, unsigned int serverport,
		std::string clientname, unsigned int proc, Enums::Protocol prot);
	TunnelID(std::string str) throw(InvalidTunnelID);
	TunnelID(const TunnelID& that);
	virtual ~TunnelID();

	// Assignment
	TunnelID& operator=(const TunnelID& that);

	// Comparison operator
	virtual bool operator==(const TunnelID& that) const;
	// Less than operator for sorting
	virtual bool operator<(const TunnelID& that) const;

	// Accessors
	unsigned int getID(void) const;
	std::string getServerIP(void) const;
	unsigned int getServerPort(void) const;
	std::string getClientName(void) const;
	unsigned int getProcID(void) const;
	Enums::Protocol getProtocol(void) const;

	// Create a representation of ourselves as a string
	virtual std::string toString(void) const;


	/**************/
	/* Predicates */
	/**************/

	// Compare 2 TunnelIDs from pointers
	struct Less {
		bool operator()(const TunnelID* const& id1, const TunnelID* const& id2) {
			return (*id1 < *id2);
		}
	};


private:

	/********/
	/* Data */
	/********/

	// Unique tunnel count ID from creating client process
	unsigned int					m_id;

	// Server end IP and port
	std::string						m_svrIP;
	unsigned int					m_svrPort;

	// Client host name
	std::string						m_cliName;

	// Client tunneler process ID
	unsigned int					m_cliProcID;

	// Protocol being tunneled
	Enums::Protocol				m_protocol;

};


// Output to a stream
std::ostream& operator<<(std::ostream& str, const TunnelID& tid);


}	// namespace

#endif

